clear
close all
clc

% Initialisation
fs          = 44100;       
T           = 1/fs; 
N           = 2048;
x           = [1,zeros(1,N)]; 	% unit impulse
f_vec       = (0:N-1) * fs/N;

% Parameters
fc          = 10000;            % cutoff frequency
G           = 15;               % Gain factor in dB

% Parameters and coefficients for boost case
V0b         = 10^(G/20);
H0b         = V0b - 1;
a_b         = (tan(2*pi*fc*T/2)-1) /( tan(2*pi*fc*T/2)+1);
% All-pass
Bb          = [a_b, 1];
Ab          = [1, a_b];

% Parameters and coefficients for cut case of high-frequency shelving
V0c         = 10^(-G/20);
H0c         = V0c-1;
a_c         = (V0c*tan(2*pi*fc*T/2)-1) / (V0c*tan(2*pi*fc*T/2)+1);
% All-pass
Bc          = [a_c, 1];
Ac          = [1, a_c];

% Coefficients for complete transfer function
% High-frequency shelving (boost)
Bb_hs       = [H0b/2*(1-a_b)+1, H0b/2*(-1+a_b)+a_b];
Ab_hs       = [1, a_b];
% High-frequency shelving (cut)
Bc_hs       = [H0c/2*(1-a_c)+1, H0c/2*(-1+a_c)+a_c];
Ac_hs       = [1, a_c];

% Filtering
y_all_b     = filter(Bb,Ab,x);              % High-frequency Shelving (Boost)
y_b         = x + H0b/2 * (x-y_all_b);      
y_all_c     = filter(Bc,Ac,x);              % High-frequency Shelving (Cut)
y_c         = x + H0c/2 * (x-y_all_c); 
y_all_bc    = filter(Bc,Ac,y_b);
y_bc        = y_b + H0c/2 * (y_b-y_all_bc); % Cascade of both

%% Task 3

figure;
subplot(3,1,1);
[Hb,W] = freqz(Bb_hs,Ab_hs);
semilogx(W/pi*fs/2,20*log10(abs(Hb)+eps))
title('Magnitude response of high-frequency shelving (boost)')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|H_B(f)| in dB')
subplot(3,1,2);
[Hc,W] = freqz(Bc_hs,Ac_hs);
semilogx(W/pi*fs/2,20*log10(abs(Hc)+eps))
title('Magnitude response of high-frequency shelving (cut)')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|H_C(f)| in dB')
subplot(3,1,3);
semilogx(W/pi*fs/2,20*log10(abs(Hb.*Hc)+eps))
title('Magnitude response of high-frequency shelving boost & cut cascade')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|H_{BC}(f)| in dB')

%% Task 4

figure;
subplot(4,1,1)
semilogx(f_vec,20*log10(abs(fft(x,N))));
title('Magnitude response of unit impulse')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|X(f)| in dB')
subplot(4,1,2)
semilogx(f_vec,20*log10(abs(fft(y_b,N))));
title('Magnitude response of high-freuency shelving (boost) output')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|Y_B(f)| in dB')
subplot(4,1,3)
semilogx(f_vec,20*log10(abs(fft(y_c,N))));
title('Magnitude response of high-freuency shelving (cut) output')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|Y_C(f)| in dB')
subplot(4,1,4)
semilogx(f_vec,20*log10(abs(fft(y_bc,N))));
title('Magnitude response of high-freuency shelving cascade output')
xlim([0 fs/2]); ylim([-G G]); grid on;
xlabel('f in Hz'); ylabel('|Y_{BC}(f)| in dB')
