clear
close all
clc

%% Second-order filter structure with quantization and noise shaping
% Example: Second-order Highpass with fc = 50 Hz (Butterworth design method)

% Parameters
N_shaping   = 1;            % 0: No noise shaping, 1: first-order, 2: second-order
f_S         = 44100;
NFFT        = 4096;
f_vec       = (0:NFFT-1) * f_S/NFFT;

% Input signal
x           = zeros(NFFT, 1);
x(1)        = 1;

% Filter coefficients
a           = [1, -1.5, 0.9];
y           = filter(1,a,x);

w_y         = 8; 
y_max       = 1;
Q_y         = (2*y_max)/(2^w_y);
y_q         = zeros(NFFT, 1);
x_hist      = zeros(length(a)-1, 1);
y_hist      = zeros(length(a)-1, 1);
if N_shaping > 0
    e_hist      = zeros(N_shaping, 1);
    if N_shaping == 1
        h_shaping       = 1;
    elseif N_shaping == 2
        h_shaping       = [2 -1];
    end
end
for n = 1 : NFFT
    % Feedback Loop
    y0          = x(n);
    for k = 2 : length(a)
        y0  	= y0 - a(k) * y_hist(k-1);
    end
    % Noise Shaping
    if N_shaping > 0
        y0  	= y0 - h_shaping * e_hist;
    end
    % Quantization
    y_q(n)      = Q_y * floor(y0/Q_y+0.5);
    if N_shaping > 0
        e_hist      = [y_q(n)-y0; e_hist(1:end-1,1)];
    end
    % Storing previous inputs and outputs
    x_hist      = [x(n); x_hist(1:end-1,1)]; 
    y_hist      = [y_q(n); y_hist(1:end-1,1)];
    clear m k
end
clear n y0 x_hist y_hist e_hist

% Error due to Qunatization
e           = y_q - y;

figure;
semilogx(f_vec, 20*log10(abs(fft(y_q,NFFT))),'b'); hold on;
semilogx(f_vec, 20*log10(abs(fft(y,NFFT))),'c'); hold off;
xlim([0 f_S/2]); ylim([-30 30]); grid on;
xlabel('Frequency in Hz'); ylabel('Magnitude in dB');
legend('|Y_q(f)|','|Y(f)|','Location','Southwest');
title('Output signals with and without quantization')

figure;
semilogx(f_vec, 20*log10(abs(fft(e,NFFT))), 'r');
xlim([0 f_S/2]); ylim([-100 0]); grid on;
xlabel('Frequency in Hz'); ylabel('Magnitude in dB');
title('Error between output signals with and without quantization')
