clear
close all
clc

%% Parameters
Nx          = 44100;                                            
nx          = 0 : Nx-1;
x           = 0.5 * cos(0.25*pi*nx) + 0.5*cos(0.7*pi*nx);

Nh          = 512;
h           = firpm(Nh-1,[0 0.4 0.7 1],[1 1 0 0]);

%% Filtering using conv
tic;
y_conv      = conv(x,h);
time_conv   = toc;

%% Filtering using sample-by-sample method
tic;
Ny          = length(x) + length(h) -1;           
Nx          = length(x);
x_buff      = zeros(1,Nh);
y_fir       = zeros(1,Ny);
for n = 1 : Nx
    x_buff      = [x(n) x_buff(1:Nh-1)];
    y_fir(n)    = x_buff * h';
end
for n = Nx+1 : Ny
    x_buff      = [0 x_buff(1:Nh-1)];
    y_fir(n)    = x_buff * h';
end
time_fir    = toc;
clear x_buff

%% Filtering using FFT method
tic;
NFFT     	= 2^(ceil(log2(Ny)));          
X        	= fft(x,NFFT);
H          	= fft(h,NFFT);
Y          	= X .* H;                                  
y_fft     	= real(ifft(Y));         
y_fft       = y_fft(1:Ny);  
time_fft    = toc;
clear NFFT X H Y

%% Filtering using FFT with overlap-add (fast convolution)
tic;
NFFT        = 1024;
L           = NFFT - Nh + 1;
Nblocks     = ceil(Nx/L);                        
x_zp        = [x, zeros(1,L*Nblocks-Nx)];                
N_overlap   = NFFT - L;                          
y_ola       = zeros(1,L*Nblocks+N_overlap);
H           = fft(h,NFFT);
for m = 1 : Nblocks
    x_start     = 1+(m-1)*L;
    x_end       = x_start + L-1;
    y_end       = x_start + NFFT-1;
    x_buff      = x_zp(x_start:x_end);
    X           = fft(x_buff,NFFT);
    Y           = X .* H;
    y0          = real(ifft(Y));
    y_ola(x_start:y_end)    = y_ola(x_start:y_end) + y0;
end
y_ola       = y_ola(1:Ny);
time_ola    = toc;
clear NFFT L Nblocks x_zp N_overlap H x_start x_end y_end x_buff X Y y0