clear
close all
clc

%% FIR filter design using Frequency Sampling

% Filter length
N  	= 8;

% Filter design
if N/2 == round(N/2) % N is even
    k           = 0 : N-1;
    % Example 1
    % H           = ones(1,N/2);
    % H(N/2+1)    = 0;
    % H(N/2+2:N)  = -1;
    % Example Book
    H(1:N/2)    = 1:-0.2:0.4;
    H(N/2+1)    = 0;
    H(N/2+2:N)  = -1 * flip(H(2:N/2));
    A           = H .* cos(2*pi*(N-1)*k/(2*N));
    B           = -H .* sin(2*pi*(N-1)*k/(2*N));
    h           = ifft(abs(H).*(A+j*B));
    % Plot
    figure;
    subplot(2,2,1);
    stem((0:N-1),abs(H));
    xlabel('k'); ylabel('|H(k)|'); grid on;
    subplot(2,2,3);
    stem((0:N-1),real(h));
    xlabel('n'); ylabel('h(n)'); grid on;
    subplot(2,2,2);
    stem((0:N-1),A);
    xlabel('k'); ylabel('Re\{H(k)\}'); grid on;
    subplot(2,2,4);
    stem((0:N-1),B)
    xlabel('k'); ylabel('Im\{H(k)\}'); grid on;

    figure;
    zplane(real(h),1);
    title('Pole/zero locations');
    xlabel('Re\{z\}'); ylabel('Im\{z\}'); grid on;
    
else % N is odd
    H                   = zeros(1,N);
    H(1:1:(N-1)/2-1)    = 1;
    H(N:-1:(N-1)/2+3)   = 1;
    M                   = (N-1)/2;
    k                   = 0:N-1;
    W                   = exp(j*2*pi/N);
    h                   = ifft(H.*(W.^(-M*k)));

    L                   = 32;
    HH                  = fft([h zeros(1,L-N)]);
    W                   = exp(j*2*pi/L);
    k                   = 0:L-1;
    A                   = HH .* W.^(M*k);
    A                   = real(A);
    w                   = k*2*pi/L;
    
    figure;
    subplot(1,2,1);
    plot(w/pi,A,2*[0:N-1]/N,H,'o');
    xlabel('\omega/\pi'); ylabel('A(\omega)'); grid on;
    subplot(1,2,2);
    stem(real(h))
    xlabel('n'); ylabel('h(n)');

    figure;
    zplane(real(h),1)
    title('Pole/zero locations');
    xlabel('Re\{z\}'); ylabel('Im\{z\}'); grid on;
end

